@@ -1,7 +1,7 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: images.h 1577 2005-07-14 13:13:30Z roms $ */
+/* $Id: images.h 2601 2007-07-14 08:49:30Z roms $ */
 
-/*  TiEmu - an TI emulator
+/*  TiEmu - Tiemu Is an EMUlator
  *
  *  Copyright (c) 2000-2001, Thomas Corvazier, Romain Lievin
  *  Copyright (c) 2001-2003, Romain Lievin
@@ -27,17 +27,23 @@
 #ifndef __TI68K_IMAGES__
 #define __TI68K_IMAGES__
 
+#include <stdint.h>
 
 /*
   Definitions
 */
 
-#define IMG_REV 2           // increase this number when changing the structure below
+#define IMG_SIGN	"TiEmu img v2.00"
+#define IMG_REV		2	// increase this number when changing the structure below
 
-// dc = don't care for rom/tib
+// Please update the docs/TiEmu_img_format.txt documentation when making changes
+// on the structure below
+
+// If this structure is modified, the SAV_REVISION number (state.c)
+// has to be incremented.
 typedef struct
 {
-	char	signature[16];	// "TiEmu v2.00" (dc)
+	char	signature[16];	// "TiEmu img v2.00" (dc)
     long    revision;       // structure revision (compatibility)
 	long	header_size;	// size of this structure and offset to pure data (dc)
 
@@ -47,11 +53,12 @@ typedef struct
 	char	has_boot;		// FLASH upgrade does not have boot
 	long	size;			// size of pure data
 	char	hw_type;		// hw1 or hw2
-    char    rom_base;       // ROM base address (MSB)
+    uint8_t rom_base;       // ROM base address (MSB)
 
     char    fill[0x40-42];  // round up struct to 0x40 bytes
 	char*	data;			// pure data (temporary use, 8 bytes)
 } IMG_INFO;
+// dc = don't care for rom/tib
 
 extern int		img_loaded;
 extern IMG_INFO img_infos;
