@@ -1,13 +1,14 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: dbg_bkpts.c 1615 2005-08-03 12:44:13Z roms $ */
+/* $Id: dbg_bkpts.c 2713 2007-12-14 21:03:03Z roms $ */
 
-/*  TiEmu - an TI emulator
+/*  TiEmu - Tiemu Is an EMUlator
  *
  *  Copyright (c) 2000-2001, Thomas Corvazier, Romain Lievin
  *  Copyright (c) 2001-2003, Romain Lievin
  *  Copyright (c) 2003, Julien Blache
  *  Copyright (c) 2004, Romain Livin
- *  Copyright (c) 2005, Romain Livin, Kevin Kofler
+ *  Copyright (c) 2005-2006, Romain Livin, Kevin Kofler
+ *  Copyright (c) 2007, Romain Livin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,17 +38,19 @@
 #include "support.h"
 #include "ti68k_int.h"
 #include "struct.h"
-#include "dbg_all.h"
+#include "dbg_wnds.h"
 #include "bkpts.h"
+#include "handles.h"
 
 static GladeXML *xml = NULL;
 static GtkWidget *wnd = NULL;
 
 enum { 
 	    COL_SYMBOL, COL_TYPE, COL_STATUS, COL_START, COL_END, COL_MODE,
+		COL_DATA, COL_FONT
 };
 #define CLIST_NVCOLS	(6)		// 7 visible columns
-#define CLIST_NCOLS		(6)		// 7 real columns
+#define CLIST_NCOLS		(8)		// 7 real columns
 
 static GtkListStore* clist_create(GtkWidget *widget)
 {
@@ -63,7 +66,7 @@ static GtkListStore* clist_create(GtkWidget *widget)
 	
 	store = gtk_list_store_new(CLIST_NCOLS,
 				G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-				G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+				G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING,
 				-1
             );
     model = GTK_TREE_MODEL(store);
@@ -78,6 +81,7 @@ static GtkListStore* clist_create(GtkWidget *widget)
 		gtk_tree_view_insert_column_with_attributes(view, -1, 
             text[i], renderer, 
             "text", i,
+			"font", COL_FONT,
 			NULL);
 	}
     
@@ -137,8 +141,12 @@ static void clist_populate(GtkListStore *store)
 		COL_START, str,
 		COL_END, "",		
         COL_MODE, BKPT_IS_TMP(addr) ? _("one-shot") : "",
+		COL_DATA, l->data,
 		-1);
-		
+/*
+		if(options3.dbg_font_type)
+			gtk_list_store_set(store, &iter, COL_FONT, options3.dbg_font_name, -1);
+*/		
 		g_free(str);
 	}
 
@@ -156,12 +164,16 @@ static void clist_populate(GtkListStore *store)
 		gtk_list_store_set(store, &iter, 
 		COL_SYMBOL, str1, 
 		COL_TYPE, ti68k_bkpt_type_to_string(BK_TYPE_EXCEPTION),
-		COL_STATUS, _("enabled"),
+		COL_STATUS, BKPT_IS_ENABLED(n) ? _("enabled") : _("disabled"),
 		COL_START, str2,
 		COL_END, "",		
 		COL_MODE, "",
+		COL_DATA, l->data,
 		-1);
-		
+/*
+		if(options3.dbg_font_type)
+			gtk_list_store_set(store, &iter, COL_FONT, options3.dbg_font_name, -1);
+*/		
 		g_free(str1);
 		g_free(str2);
 	}
@@ -184,8 +196,12 @@ static void clist_populate(GtkListStore *store)
 			COL_START, str,
 			COL_END, "",
             COL_MODE, ti68k_bkpt_mode_to_string(BK_TYPE_ACCESS, bkpts_memacc_rw[i]),
+			COL_DATA, l->data,
 			-1);
-			
+/*
+			if(options3.dbg_font_type)
+				gtk_list_store_set(store, &iter, COL_FONT, options3.dbg_font_name, -1);
+*/			
 			g_free(str);
 		}
 	}
@@ -209,12 +225,41 @@ static void clist_populate(GtkListStore *store)
 			COL_START, str1,
 			COL_END, str2,
             COL_MODE, ti68k_bkpt_mode_to_string(BK_TYPE_RANGE, bkpts_memrng_rw[i]),
+			COL_DATA, l->data,
 			-1);
-			
+/*
+			if(options3.dbg_font_type)
+				gtk_list_store_set(store, &iter, COL_FONT, options3.dbg_font_name, -1);
+*/			
 			g_free(str1);
 			g_free(str2);
 		}
 	}
+	
+	// Bit change breakpoints
+	for(l = bkpts.bits; l != NULL; l = g_list_next(l))
+	{
+		ADDR_BIT *s = l->data;
+		gchar *str;
+
+		str = g_strdup_printf("0x%06x", BKPT_ADDR(s->addr));
+
+		gtk_list_store_append(store, &iter);
+			gtk_list_store_set(store, &iter, 
+			COL_SYMBOL, str, 
+			COL_TYPE, ti68k_bkpt_type_to_string(BK_TYPE_BIT),
+			COL_STATUS, BKPT_IS_ENABLED(s->addr) ? _("enabled") : _("disabled"),
+			COL_START, str,
+			COL_END, "",
+            COL_MODE, "",
+			COL_DATA, l->data,
+			-1);
+/*
+			if(options3.dbg_font_type)
+				gtk_list_store_set(store, &iter, COL_FONT, options3.dbg_font_name, -1);
+*/
+		g_free(str);
+	}
 
 	// parse list to merge informations: ugly/heavy code !
 	for(valid = gtk_tree_model_get_iter_first(model, &iter);
@@ -252,7 +297,7 @@ static void clist_populate(GtkListStore *store)
 				{
 					if(!strcmp(row_text[COL_START], row_text2[COL_START]) && !strcmp(row_text[COL_END], row_text2[COL_END]))
 					{
-						if( (mode & BK_READ) && (mode2 & BK_WRITE) || (mode & BK_WRITE) && (mode2 & BK_READ) )
+						if( ((mode & BK_READ) && (mode2 & BK_WRITE)) || ((mode & BK_WRITE) && (mode2 & BK_READ)) )
 						{
 							const gchar *new_str;
 							
@@ -291,8 +336,12 @@ static void clist_populate(GtkListStore *store)
 		COL_START, str,
 		COL_END, "",		
         COL_MODE, BKPT_IS_TMP(addr) ? _("one-shot") : "",
+		COL_DATA, l->data,
 		-1);
-		
+/*
+		if(options3.dbg_font_type)
+			gtk_list_store_set(store, &iter, COL_FONT, options3.dbg_font_name, -1);
+*/		
 		g_free(str);
 	}
 }
@@ -311,13 +360,12 @@ GtkWidget* dbgbkpts_create_window(void)
 		(tilp_paths_build_glade("dbg_bkpts-2.glade"), "dbgbkpts_window",
 		 PACKAGE);
 	if (!xml)
-		g_error(_("%s: GUI loading failed !\n"), __FILE__);
+		g_error(_("%s: GUI loading failed!\n"), __FILE__);
 	glade_xml_signal_autoconnect(xml);
 	
 	dbox = glade_xml_get_widget(xml, "dbgbkpts_window");
-#ifdef WND_TRANSIENT
-	gtk_window_set_transient_for(GTK_WINDOW(dbox), GTK_WINDOW(main_wnd));
-#endif
+	if(options3.transient)
+		gtk_window_set_transient_for(GTK_WINDOW(dbox), GTK_WINDOW(main_wnd));
 	
 	data = glade_xml_get_widget(xml, "treeview1");
     store = clist_create(data);
@@ -340,14 +388,9 @@ GtkWidget* dbgbkpts_display_window(void)
 		gtk_window_iconify(GTK_WINDOW(wnd));
 #endif
 
-	gtk_list_store_clear(store);
-    clist_populate(store);
-
 	if(!GTK_WIDGET_VISIBLE(dbgw.bkpts) && !options3.bkpts.closed)
 		gtk_widget_show(wnd);
 
-	display_dbgcause_dbox2(glade_get("statusbar1"));
-
 	return wnd;
 }
 
@@ -357,9 +400,15 @@ void dbgbkpts_refresh_window(void)
 	{
 		gtk_list_store_clear(store);
 		clist_populate(store);
+
+		display_dbgcause_dbox2(glade_get("statusbar1"));
 	}
 }
 
+void dbgbkpts_erase_context(void)
+{
+	display_dbgcause_dbox2(glade_get("statusbar1"));
+}
 
 static GtkWidget* display_popup_menu(void)
 {
@@ -450,6 +499,10 @@ dbgbkpts_button2_clicked                     (GtkButton       *button,
 			sscanf(row_text[COL_SYMBOL], "#%04x", &n);
 			ti68k_bkpt_del_pgmentry((uint16_t)n);
 			break;
+		case BK_TYPE_BIT:
+			mode = ti68k_string_to_bkpt_mode(row_text[COL_MODE]);
+            sscanf(row_text[COL_START], "%x", &min);
+            ti68k_bkpt_del_bits(min);
         }
         g_strfreev(row_text);
     }
@@ -498,28 +551,53 @@ dbgbkpts_button3_clicked                     (GtkButton       *button,
         {
         case BK_TYPE_CODE:
             sscanf(row_text[COL_START], "%x", &min);
-            ti68k_bkpt_set_address(BKPT_ADDR(min), BKPT_DISABLE(min));
+            {
+              uint32_t addr = BKPT_ADDR(min);
+              ti68k_bkpt_set_address(addr, BKPT_DISABLE(min));
+            }
             break;
         case BK_TYPE_EXCEPTION:
             sscanf(row_text[COL_SYMBOL], "#%i", &n);
-            ti68k_bkpt_set_exception(BKPT_ADDR(n), BKPT_DISABLE(n));
+            {
+              uint32_t addr = BKPT_ADDR(n);
+              ti68k_bkpt_set_exception(addr, BKPT_DISABLE(n));
+            }
             break;
         case BK_TYPE_ACCESS:
             mode = ti68k_string_to_bkpt_mode(row_text[COL_MODE]);
             sscanf(row_text[COL_START], "%x", &min);
-            ti68k_bkpt_set_access(BKPT_ADDR(min), mode, BKPT_DISABLE(min));
+            {
+              uint32_t addr = BKPT_ADDR(min);
+              ti68k_bkpt_set_access(addr, mode, BKPT_DISABLE(min));
+            }
             break;
         case BK_TYPE_RANGE:
             mode = ti68k_string_to_bkpt_mode(row_text[COL_MODE]);
             sscanf(row_text[COL_START], "%x", &min);
             sscanf(row_text[COL_END], "%x", &max);
-            ti68k_bkpt_set_range(BKPT_ADDR(min), BKPT_ADDR(max), mode,
-								BKPT_DISABLE(min), BKPT_DISABLE(max));
+            {
+              uint32_t addr1 = BKPT_ADDR(min);
+              uint32_t addr2 = BKPT_ADDR(max);
+              ti68k_bkpt_set_range(addr1, addr2, mode,
+                                   BKPT_DISABLE(min), BKPT_DISABLE(max));
+            }
             break;
-		case BK_TYPE_PGMENTRY:
-			//sscanf(row_text[COL_SYMBOL], "#%04x", &n);
-			//ti68k_bkpt_set_pgmentry(BKPT_ADDR(n), BKPT_DISABLE(n));
+        case BK_TYPE_PGMENTRY:
+            //sscanf(row_text[COL_SYMBOL], "#%04x", &n);
+            //{
+            //  uint32_t addr = BKPT_ADDR(n);
+            //  ti68k_bkpt_set_pgmentry(addr, BKPT_DISABLE(n));
+            //}
+            break;
+		case BK_TYPE_BIT:
+			mode = ti68k_string_to_bkpt_mode(row_text[COL_MODE]);
+            sscanf(row_text[COL_START], "%x", &min);
+            {
+              uint32_t addr = BKPT_ADDR(min);
+              ti68k_bkpt_set_bits(addr, BKPT_DISABLE(addr));
+            }
 			break;
+
         }
         g_strfreev(row_text);
     }
@@ -567,27 +645,51 @@ dbgbkpts_button4_clicked                     (GtkButton       *button,
         {
         case BK_TYPE_CODE:
             sscanf(row_text[COL_START], "%x", &min);
-            ti68k_bkpt_set_address(BKPT_ADDR(min), BKPT_ENABLE(min));
+            {
+              uint32_t addr = BKPT_ADDR(min);
+              ti68k_bkpt_set_address(addr, BKPT_ENABLE(min));
+            }
             break;
         case BK_TYPE_EXCEPTION:
             sscanf(row_text[COL_SYMBOL], "#%i", &n);
-            ti68k_bkpt_set_exception(BKPT_ADDR(n), BKPT_ENABLE(n));
+            {
+              uint32_t addr = BKPT_ADDR(n);
+              ti68k_bkpt_set_exception(addr, BKPT_ENABLE(n));
+            }
             break;
         case BK_TYPE_ACCESS:
             mode = ti68k_string_to_bkpt_mode(row_text[COL_MODE]);
             sscanf(row_text[COL_START], "%x", &min);
-            ti68k_bkpt_set_access(BKPT_ADDR(min), mode, BKPT_ENABLE(min));
+            {
+              uint32_t addr = BKPT_ADDR(min);
+              ti68k_bkpt_set_access(addr, mode, BKPT_ENABLE(min));
+            }
             break;
         case BK_TYPE_RANGE:
             mode = ti68k_string_to_bkpt_mode(row_text[COL_MODE]);
             sscanf(row_text[COL_START], "%x", &min);
             sscanf(row_text[COL_END], "%x", &max);
-            ti68k_bkpt_set_range(BKPT_ADDR(min), BKPT_ADDR(max), mode,
-								BKPT_ENABLE(min), BKPT_ENABLE(max));
+            {
+              uint32_t addr1 = BKPT_ADDR(min);
+              uint32_t addr2 = BKPT_ADDR(max);
+              ti68k_bkpt_set_range(addr1, addr2, mode,
+                                   BKPT_ENABLE(min), BKPT_ENABLE(max));
+            }
             break;
-		case BK_TYPE_PGMENTRY:
-			//sscanf(row_text[COL_SYMBOL], "#%04x", &n);
-			//ti68k_bkpt_set_pgmentry(BKPT_ADDR(n), BKPT_ENABLE(n));
+        case BK_TYPE_PGMENTRY:
+            //sscanf(row_text[COL_SYMBOL], "#%04x", &n);
+            //{
+            //  uint32_t addr = BKPT_ADDR(n);
+            //  ti68k_bkpt_set_pgmentry(addr, BKPT_ENABLE(n));
+            //}
+            break;
+		case BK_TYPE_BIT:
+			mode = ti68k_string_to_bkpt_mode(row_text[COL_MODE]);
+            sscanf(row_text[COL_START], "%x", &min);
+            {
+              uint32_t addr = BKPT_ADDR(min);
+              ti68k_bkpt_set_bits(addr, BKPT_ENABLE(addr));
+            }
 			break;
         }
         g_strfreev(row_text);
@@ -620,7 +722,7 @@ dbgbkpts_button5_clicked                     (GtkButton       *button,
 		GtkTreeIter iter;
 		GtkTreePath *path = l->data;
         gchar** row_text = g_malloc0((CLIST_NVCOLS + 1) * sizeof(gchar *));
-        uint32_t type, addr;
+        uint32_t type, min, n, addr;
 			
 		gtk_tree_model_get_iter(model, &iter, path);
 		gtk_tree_model_get(model, &iter, 
@@ -632,13 +734,38 @@ dbgbkpts_button5_clicked                     (GtkButton       *button,
             -1);
 		
 		type = ti68k_string_to_bkpt_type(row_text[COL_TYPE]);
-		if(type == BK_TYPE_CODE)
-		{
-			// get address
-            sscanf(row_text[COL_START], "%x", &addr);
+		switch(type)
+        {
+        case BK_TYPE_CODE:
+            sscanf(row_text[COL_START], "%x", &min);
+			addr = BKPT_ADDR(min);
+          	dbgcode_disasm_at(addr);
+            break;
+        case BK_TYPE_EXCEPTION:
+            sscanf(row_text[COL_SYMBOL], "#%i", &n);
+			dbgcode_disasm_at(mem_rd_long(4*n));
+            break;
+        case BK_TYPE_ACCESS:
+		case BK_TYPE_RANGE:
+		case BK_TYPE_BIT:
+            sscanf(row_text[COL_START], "%x", &min);
+            addr = BKPT_ADDR(min);
+            dbgmem_add_tab(addr);
+            break;
+        case BK_TYPE_PGMENTRY:
+			{
+				gpointer data;
+				uint16_t handle, offset;
 
-			// populate code
-			dbgcode_disasm_at(addr);
+				gtk_tree_model_get(model, &iter, COL_DATA, &data, -1);
+
+				handle = GPOINTER_TO_INT(data) >> 16;
+				offset = GPOINTER_TO_INT(data) & 0xffff;
+				addr = heap_deref(handle) + offset;
+
+				dbgcode_disasm_at(addr);
+			}
+            break;
         }
         g_strfreev(row_text);
     }
@@ -670,6 +797,7 @@ on_treeview2_button_press_event        (GtkWidget       *widget,
 	    gint cx, cy;
 		gchar** row_text = g_malloc0((CLIST_NVCOLS + 1) * sizeof(gchar *));
 		uint32_t type;
+		gpointer data;
 		
         ret = gtk_tree_view_get_path_at_pos(view, tx, ty, &path, &column, &cx, &cy);
         if(ret == FALSE)
@@ -685,6 +813,7 @@ on_treeview2_button_press_event        (GtkWidget       *widget,
             COL_START, &row_text[COL_START], 
             COL_END, &row_text[COL_END],
             COL_MODE, &row_text[COL_MODE],
+			COL_DATA, &data,
             -1);
 		
         type = ti68k_string_to_bkpt_type(row_text[COL_TYPE]);
@@ -726,7 +855,7 @@ on_treeview2_button_press_event        (GtkWidget       *widget,
 				ti68k_bkpt_add_access(new_min, new_mode);
 				//ti68k_bkpt_set_access(old_min, mode, new_min);
 			}
-			else if(new_type = BK_TYPE_RANGE)
+			else if(new_type == BK_TYPE_RANGE)
 			{
 				ti68k_bkpt_del_range(old_min, old_max, old_mode);
 				ti68k_bkpt_add_range(new_min, new_max, new_mode);
@@ -734,6 +863,18 @@ on_treeview2_button_press_event        (GtkWidget       *widget,
 			}
 			dbgbkpts_refresh_window();
 		}
+		if(type == BK_TYPE_BIT)
+		{
+			uint32_t old_addr;
+			ADDR_BIT *s = (ADDR_BIT *)data;
+
+			sscanf(row_text[COL_START], "%x", &old_addr);
+
+			if(dbgbits_display_dbox(&s->addr, &s->checks, &s->states) == -1)
+				return TRUE;
+
+			dbgbkpts_refresh_window();
+		}
 
 		g_strfreev(row_text);
                 
@@ -743,6 +884,20 @@ on_treeview2_button_press_event        (GtkWidget       *widget,
     return FALSE;
 }
 
+GLADE_CB void
+dbgbkpts_bit_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	uint32_t addr = 0;
+	uint8_t checks = 0, states = 0xff;
+
+	if(dbgbits_display_dbox(&addr, &checks, &states) != 0)
+		return;
+
+	ti68k_bkpt_add_bits(addr, checks, states);
+
+	dbgbkpts_refresh_window();
+}
 
 GLADE_CB void
 dbgbkpts_data_activate                    (GtkMenuItem     *menuitem,
