@@ -1,7 +1,7 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: dbg_code.h 1455 2005-05-31 18:38:03Z roms $ */
+/* $Id: dbg_code.h 2268 2006-11-06 17:18:51Z roms $ */
 
-/*  TiEmu - an TI emulator
+/*  TiEmu - Tiemu Is an EMUlator
  *
  *  Copyright (c) 2000-2001, Thomas Corvazier, Romain Lievin
  *  Copyright (c) 2001-2003, Romain Lievin
@@ -26,8 +26,11 @@
 
 #include <gtk/gtk.h>
 
+extern gint reset_disabled;
+
 GtkWidget* dbgcode_create_window(void);
 GtkWidget* dbgcode_display_window(void);
 void dbgcode_refresh_window(void);
+int dbgcode_quit_enabled(void);
 
 void dbgcode_disasm_at(uint32_t addr);
