@@ -7,7 +7,7 @@
  *  Copyright (c) 2001-2003, Romain Lievin
  *  Copyright (c) 2003, Julien Blache
  *  Copyright (c) 2004, Romain Livin
- *  Copyright (c) 2005-2007, Romain Livin, Kevin Kofler
+ *  Copyright (c) 2005-2008, Romain Livin, Kevin Kofler
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -265,6 +265,10 @@ on_dockmode1_activate                  (GtkMenuItem     *menu_item,
 	engine_start();
 #endif
 	gtk_debugger_close();
+	// Stop the engine before calling gtk_main_quit.
+	// Otherwise, it will keep running even when it is supposed to have
+	// been stopped by the debugger.
+	engine_stop();
 	if(options3.dbg_dock)
 		gtk_widget_destroy(dbgw.dock);
 
@@ -714,4 +718,4 @@ void profile_stop(const char *str)
 {
 	printf("Duration (%s): %i ms\n", str, TO_CURRENT(profile));
 }
-*/
\ No newline at end of file
+*/
